SystemOrganization addCategory: #Array2D!

TestCase subclass: #Array2DTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Array2D'!

!Array2DTest methodsFor: 'testing' stamp: 'md 2/14/2004 18:14'!
testRowAndColumnValuesDo
	| foo |

	 foo _ Array2D width: 5 height: 20.   
	 self shouldnt: [foo rowAndColumnValuesDo: [ :row :col :val ]] raise: Error.
! !

ArrayedCollection subclass: #Array2D
	instanceVariableNames: 'width contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Array2D'!

!Array2D commentStamp: '<historical>' prior: 0!
A simple 2D-Array implementation. 

Neither storing nor sorting (otherwise inherited from ArrayedCollection) will work. Neither comparing nor most accessing mehods inherited from Sequenceable collection will work.  Actually, it's a bad idea to inherit this class from collection at all!!!

!Array2D class methodsFor: 'instance creation' stamp: 'sma 4/22/2000 18:10'!
width: width height: height
	^ self basicNew width: width height: height type: Array! !

!Array2D methodsFor: 'accessing' stamp: 'sma 4/22/2000 18:21'!
height
    "Answer the receiver's second dimension."

	^ contents size // width! !

!Array2D methodsFor: 'accessing' stamp: 'sma 4/22/2000 18:22'!
width
	"Answer the receiver's first dimension."

	^ width! !

!Array2D methodsFor: 'copying' stamp: 'sma 4/22/2000 18:37'!
copy
	^ super copy setContents: contents copy! !

!Array2D methodsFor: 'enumeration' stamp: 'je 3/23/2003 17:21'!
rowAndColumnValuesDo: aBlock
	1 to: self width do: [:col |
		1 to: self height do: [:row |
			aBlock value: row value: col value: (self at: col at: row)]]! !

!Array2D class methodsFor: 'instance creation' stamp: 'sma 4/22/2000 18:11'!
new: size
	self error: 'Use >>self width: x height: y<< instead'! !

!Array2D methodsFor: 'converting' stamp: 'sma 4/22/2000 18:38'!
asArray
	^ contents copy! !

!Array2D methodsFor: 'private' stamp: 'sma 4/22/2000 18:37'!
setContents: aCollection
	contents _ aCollection! !

!Array2D methodsFor: 'private' stamp: 'sma 4/22/2000 18:24'!
extent: extent fromArray: anArray
	"Load receiver up from a 1-D array.  X varies most quickly.  6/20/96 tk"

	extent x * extent y = anArray size ifFalse:
		[^ self error: 'dimensions don''t match'].
	width _ extent x.
	contents _ anArray! !

!Array2D methodsFor: 'accessing' stamp: 'sma 4/22/2000 18:16'!
at: x at: y
	"Answer the element at index x,y."

	^ contents at: (self indexX: x y: y)! !

!Array2D methodsFor: 'accessing rows/columns' stamp: 'sma 4/22/2000 18:30'!
atRow: y put: aCollection
	"Put in a whole row."

	aCollection size = self width ifFalse: [self error: 'wrong row size'].
	aCollection doWithIndex: [:value :x | self at: x at: y put: value].
	^ aCollection! !

!Array2D methodsFor: 'accessing rows/columns' stamp: 'sma 4/22/2000 18:27'!
atCol: x
	"Answer a whole column."

	| column |
	column _ contents class new: self height.
	1 to: self height do: [:index | column at: index put: (self at: x at: index)].
	^ column! !

!Array2D methodsFor: 'private' stamp: 'sma 4/22/2000 18:13'!
width: x height: y type: collectionClass
	"Set the number of elements in the first and second dimension.
	collectionClass can be Array or String or ByteArray."

	contents == nil ifFalse:
		[self error: 'No runtime size change yet'].
		"later move all the elements to the new sized array"
	width _ x.
	contents _ collectionClass new: x * y! !

!Array2D methodsFor: 'accessing rows/columns' stamp: 'sma 4/22/2000 18:27'!
atRow: y
    "Answer a whole row."

	(y < 1 or: [y > self height]) ifTrue: [self errorSubscriptBounds: y].
	^ contents copyFrom: y - 1 * width + 1 to: y * width! !

!Array2D methodsFor: 'private' stamp: 'sma 4/22/2000 18:16'!
indexX: x y: y
	(x < 1 or: [x > width]) ifTrue: [self errorSubscriptBounds: x].
	^ y - 1 * width + x! !

!Array2D methodsFor: 'enumeration' stamp: 'sma 4/22/2000 18:39'!
rowsAndColumnsDo: aBlock
	1 to: self width do: [:col |
		1 to: self height do: [:row |
			aBlock value: row value: col]]! !

!Array2D class methodsFor: 'instance creation' stamp: ''!
new
	"Override ArrayedCollection.  6/20/96 tk"
	^ self basicNew! !

!Array2D methodsFor: 'accessing rows/columns' stamp: 'sma 4/22/2000 18:30'!
atCol: x put: aCollection
	"Put in a whole column."

	aCollection size = self height ifFalse: [self error: 'wrong column size'].
	aCollection doWithIndex: [:value :y | self at: x at: y put: value].
	^ aCollection! !

!Array2D methodsFor: 'accessing' stamp: 'sma 4/22/2000 18:20'!
atAllPut: anObject
	"Put anObject at every one of the receiver's indices."

	contents atAllPut: anObject! !

!Array2D methodsFor: 'accessing' stamp: 'sma 4/22/2000 18:22'!
extent
	"Answer the receiver's dimensions as point."

	^ self width @ self height! !

!Array2D class methodsFor: 'instance creation' stamp: 'sma 4/22/2000 18:10'!
width: width height: height type: collectionClass
	^ self basicNew width: width height: height type: collectionClass! !

!Array2D methodsFor: 'enumeration' stamp: 'sma 4/22/2000 18:14'!
do: aBlock
	"Iterate with X varying most quickly.  6/20/96 tk"

	contents do: aBlock! !

!Array2D methodsFor: 'accessing' stamp: 'sma 4/22/2000 18:20'!
at: x at: y add: value
	"Add value (using #+) to the existing element at index x,y."

	| index |
	index _ self indexX: x y: y.
	^ contents at: index put: (contents at: index) + value! !

!Array2D class methodsFor: 'instance creation' stamp: 'sma 4/22/2000 18:40'!
extent: aPoint
	^ self width: aPoint x height: aPoint y! !

!Array2D methodsFor: 'accessing' stamp: 'sma 4/22/2000 18:22'!
size
	^ contents size! !

!Array2D methodsFor: 'accessing' stamp: 'sma 4/22/2000 18:20'!
at: x at: y put: value
	"Store value at index x,y and answer it."

	^ contents at: (self indexX: x y: y) put: value! !
